﻿/*:
 * @target MZ
 * @plugindesc This plugin automatically removes incapacitated party members from the party.
 * @author Malomi
 * @url https://gendaigoyaku.com/
 * 
 * @param ImmortalActors
 * @text A character that does not leave the party.
 * @type actor[]
 * @desc Set characters who do not leave the party even when incapacitated by default.
 * @default []
 * 
 * @param LeaveMessage
 * @text The message displayed when leaving the party.
 * @desc The message displayed when an incapacitated character is removed from the party."%1"will be replaced with the actor’s name.
 * @default %1 has perished.
 * 
 * @command addImmortalActor
 * @text Add characters who do not leave the party.
 * @desc Add characters who do not leave the party even when incapacitated.
 * 
 * @arg actorId
 * @type actor
 * @text actorID
 * @desc Specify the ID of the actor to be set as a character who does not leave the party.
 * 
 * @command removeImmortalActor
 * @text Remove non-leaving characters.
 * @desc Remove characters who do not leave the party even when incapacitated.
 * 
 * @arg actorId
 * @type actor
 * @text actorID
 * @desc Specify the ID of the actor to be removed from the non-leaving characters.
 * 
 * @help
 * ----------------------------------------------------------------------------
 * RemoveDeadMembers.js
 * ----------------------------------------------------------------------------
 * This plugin automatically removes party members from the party
 *  when they become incapacitated during combat.
 * 
 * You can also set characters who do not leave the party even when they die.
 * Configured characters will remain in the party even when incapacitated.
 * 
 * You can specify characters who do not leave the party by default.
 * Additionally, you can use plugin commands to add or remove non-leaving characters
 *  through in-game events.
 * 
 * Plugin Command:
 * - addImmortalActor: Add non-leaving characters by specifying the actor ID.
 * - removeImmortalActor: Remove non-leaving characters by specifying the actor ID.
 * ----------------------------------------------------------------------------
 * License:
 * This is the MIT License. 
 * It can be used for both commercial and non-commercial purposes.
 * ----------------------------------------------------------------------------
 */

/*:ja
 * @target MZ
 * @plugindesc 戦闘不能のパーティーメンバーを自動でパーティーから外します。
 * @author まろみ
 * @url https://gendaigoyaku.com/
 * 
 * @param ImmortalActors
 * @text 離脱しないキャラクター
 * @type actor[]
 * @desc 初期設定で戦闘不能になってもパーティーから離脱しないキャラクターを設定します。
 * @default []
 * 
 * @param LeaveMessage
 * @text 離脱メッセージ
 * @desc 戦闘不能でパーティーから外れるときに表示するメッセージ。%1はアクター名になります。
 * @default %1は息絶えた。
 * 
 * @command addImmortalActor
 * @text 離脱しないキャラ追加
 * @desc 戦闘不能になってもパーティーから離脱しないキャラクターを追加します。
 * 
 * @arg actorId
 * @type actor
 * @text アクターID
 * @desc 離脱しないキャラクターとして設定するアクターのIDを指定します。
 * 
 * @command removeImmortalActor
 * @text 離脱しないキャラ解除
 * @desc 戦闘不能になってもパーティーから離脱しないキャラクターを解除します。
 * 
 * @arg actorId
 * @type actor
 * @text アクターID
 * @desc 離脱しないキャラクターから解除するアクターのIDを指定します。
 * 
 * @help
 * ----------------------------------------------------------------------------
 * RemoveDeadMembers.js
 * ----------------------------------------------------------------------------
 * このプラグインは、戦闘中にパーティーメンバーが戦闘不能になると
 * 自動的にそのメンバーをパーティーから外す機能を提供します。
 * 
 * また、死んでもパーティーから外れないキャラクターを設定できます。
 * 設定したキャラクターは戦闘不能でもパーティーに残ります。
 * 
 * パーティーから外れないキャラクターは初期設定で指定できます。
 * プラグインコマンドを使用することでも、離脱しないキャラクターをゲーム内イベント
 * から追加・解除できます。
 * 
 * プラグインコマンド:
 * - 離脱しないキャラ追加: アクターIDを指定して、離脱しないキャラクターを追加します。
 * - 離脱しないキャラ解除: アクターIDを指定して、離脱しないキャラクターを解除します。
 * ----------------------------------------------------------------------------
 * 利用規約:
 * 商用・非商用問わず利用可能です。
 * ----------------------------------------------------------------------------
 */

(() => {
    const parameters = PluginManager.parameters('RemoveDeadMembers');
    const immortalActors = new Set(JSON.parse(parameters['ImmortalActors'] || '[]').map(Number));
    const leaveMessage = String(parameters['LeaveMessage'] || '%1は息絶えた。');

    const _Game_Battler_die = Game_Battler.prototype.die;
    Game_Battler.prototype.die = function () {
        if (this.isActor() && !this._leftParty) {
            this._leftParty = true;
            const actorId = this.actorId();
            if (!immortalActors.has(actorId)) {
                const actorName = this.name();
                const message = leaveMessage.replace('%1', actorName);
                $gameMessage.add(message);
                $gameParty.removeActor(actorId);
            }
        }
        _Game_Battler_die.call(this);
    };

    PluginManager.registerCommand('RemoveDeadMembers', 'addImmortalActor', args => {
        const actorId = Number(args.actorId);
        if (!immortalActors.has(actorId)) {
            immortalActors.add(actorId);
        }
    });

    PluginManager.registerCommand('RemoveDeadMembers', 'removeImmortalActor', args => {
        const actorId = Number(args.actorId);
        if (immortalActors.has(actorId)) {
            immortalActors.delete(actorId);
        }
    });
})();
